from setuptools import setup, find_packages

setup(
    name='dola',
    author='Zeqiang Lai',
    author_email='laizeqiang@outlook.com',
    packages=find_packages(),
    version='0.0.4',
    include_package_data=True,
)
